!DIR$ FIXEDFORMLINESIZE:132
      subroutine job_plwc_sur_cyl()
      use math_blocks
      use rte_blocks
      use header_types
      implicit none
c
c----------------- Local variables -------------------------------
c
      integer*8 iw, ns, nv, na, npT, npR, life

      logical*4 absorbed, fail

      real*4    perc
      real*4    zw, dz, sx, sy, sz
      real*4    cth_ir, sth_ir, cph_ir, sph_ir, del_ir

      integer*4 ir

 400  format(1H+, a7, f5.1, a10)
 401  format(a44, f8.4, a3)
 402  format(a46,i14)

      if(SourceType .ne. 'PLWC') then
         write(*,*) 'JOB_PLWC_SUR_CYL unexpected error: SourceType=/=PLWC'
         return
      end if

      if(Domain .ne. 'SUR') then
         write(*,*) 'JOB_PLWC_SUR_CYL unexpected error: Domain=/=SUR'
         return
      end if

      if(.not. Use_Symmetry) then
         write(*,*) 'JOB_PLWC_SUR_CYL unexpected error: Use_Symmetry=F'
         return
      end if

      write(*,*) 'Note: for this job, parameter NCOMP is ignored; all quantities are computed'

c
c---------- Check memory and allocate arrays -----------------
c     

      call am_plws_sur_cyl(fail)
      if(fail) then
         write(*,*) 'JOB_PLWC_SUR_CYL: Memory could not be allocated; exiting'
         return
      else
         write(*,401) 'JOB_PLWC_SUR_CYL Memory used:', sngl(nmem*nano), ' Gb'
      end if

c
c----------------- Initialization -------------------------------
c

      call discard_rnums()  ! Discard first NR random numbers in each stream
      call make_rnums()     ! Random streams first-run

      nv = 0
      ir = 0

      na  = 0 
      npT = 0
      npR = 0

c
c---------- Loop over random walkers -------------------------------
c 

      do 100, iw=1,nw

      if(mod(iw,nc) .eq. 0) then
         perc = 100.0*float(iw)/float(nw)
         write(6,400) '       ', perc, '% complete'
      end if

c............ a) Initial position

      zw = z0

c............ b) First step

      ir = ir + 1

      del_ir = del(ir)

      sx = s0x
      sy = s0y
      sz = s0z

      dz = del_ir*sz
      zw = zw + dz

      if(bin(ir) .eq. 1) then
         absorbed = .true.
      else
         absorbed = .false.
      end if
      
      if(ir .eq. nr) then
         nv = nv + 1
         call make_rnums()
         ir = 0
      end if

      if(zw .lt. zsmin) then
         npR = npR+1
         call upd_plws_sur_cyl_R(sz)
         goto 100
      end if

      if(zw .gt. zsmax) then
         npT = npT + 1
         call upd_plws_sur_cyl_T(sz)
         goto 100
      end if

      if(absorbed) then
         na = na + 1
         goto 100
      end if

c............ c) The rest of photon life (after the first step)

 10   continue
      
      ir = ir + 1

      cth_ir = cth(ir)
      sth_ir = sth(ir)
      cph_ir = cph(ir)
      sph_ir = sph(ir)
      del_ir = del(ir)

      call rotate_s(cth_ir,sth_ir,cph_ir,sph_ir,sx,sy,sz)

      dz = del_ir*sz
      zw = zw + dz

      if(bin(ir) .eq. 1) then
         absorbed = .true.
      else
         absorbed = .false.
      end if
      
      if(ir .eq. nr) then

         nv = nv + 1
         call make_rnums()
         ir = 0

      end if

      if(zw .lt. zsmin) then
         npR = npR+1
         call upd_plws_sur_cyl_R(sz)
         goto 100
      end if

      if(zw .gt. zsmax) then 
         npT = npT + 1
         call upd_plws_sur_cyl_T(sz)
         goto 100
      end if

      if(absorbed) then
         na = na + 1
         goto 100
      end if

      goto 10

 100  continue
      close(unit=6)

c
c---------- End of random walks ---------------------------
c

      call stop_rstreams() ! Deinitialize random streams str1,str2,str3,str4

      ns   = (nv-1)*int8(nr) + int8(ir)
      life = nint(dfloat(ns)/dfloat(nw))

      write(*,*)   '------------------------------------------------------------'
      write(*,402) '                      Total number of photons:', nw
      write(*,402) '                   Number of absorbed photons:', na
      write(*,402) 'Number of photons that exited through z=zsmax:', npT
      write(*,402) 'Number of photons that exited through z=zsmin:', npR
      write(*,402) '          Number of steps made by all walkers:', ns
      write(*,402) 'Number of steps per one walker (average life):', life
      write(*,402) '                       Number of vectors used:', nv
      write(*,*)   '------------------------------------------------------------'
     
      call wd_plws_sur_cyl()

      stop
      end
