!DIR$ FIXEDFORMLINESIZE:132
      subroutine check_pars(errcode)
      use math_blocks
      use rte_blocks
      implicit none
      integer*4, intent(out) :: errcode

      integer*8 nr8, nc8
      integer*8 lx8, ly8, lz8, lrho8
      integer*4 ncomp, info
      real*4    s0, gmem_tot
      real*4    fun_exp

      errcode = 0
c
c---------- (1) Reading input parameters -----------------------------
c
c.......... (1.a) Reading RTE parameters
c 
      open(unit=70,file='rte.par',status='old',err=1000)
         read(70,*  ,err=1001,end=1001) mu_a, mu_s
         read(70,*  ,err=1002,end=1002) g
         read(70,*  ,err=1003,end=1003) zsmin, zsmax
         read(70,*  ,err=1004,end=1004) xbmin, ybmin, zbmin
         read(70,*  ,err=1005,end=1005) xbmax, ybmax, zbmax
         read(70,*  ,err=1006,end=1006) lx8, ly8, lz8
         read(70,*  ,err=1007,end=1007) rhomax, Lrho8
         read(70,*  ,err=1008,end=1008) z0
         read(70,*  ,err=1009,end=1009) s0x, s0y, s0z
         read(70,*  ,err=1010,end=1010) Nscth, Nsphi
         read(70,*  ,err=1011,end=1011) ncomp
         read(70,500,err=1012,end=1012) SourceType
         read(70,500,err=1013,end=1013) PhaseFunction
         read(70,500,err=1014,end=1014) Domain
         read(70,500,err=1015,end=1015) Units
         read(70,501,err=1016,end=1016) Use_Symmetry
      close(unit=70)
 500  format(a8)
 501  format(L)

c
c......... (1.b) Checking RTE parameters for sanity
c

c.......... (1.b.i) Domain ...................

      if     (Domain .eq. 'VOL') then
         continue
      else if(Domain .eq. 'SUR') then
         continue
      else
         write(*,*) 'Error on input [rte.par]: incorrect Domain: ', Domain
	 errcode = 1
	 return
      end if

c.......... (1.b.ii) Optical parameters ...................

      if(mu_a .lt. rzero) then
         write(*,*) 'Error on input [rte.par]: mu_a<0'
         errcode = 2
         return
      end if

      if(mu_s .lt. rzero) then
         write(*,*) 'Error on input [rte.par]: mu_s<0'
         errcode = 3
         return
      end if

      if(abs(g) .gt. ru) then
         write(*,*) 'Error on input [rte.par]: |g|>1'
         errcode = 4
      end if

c.......... (1.b.iii) Dimensions ...................

      if(xbmax .le. xbmin) then
         write(*,*) 'Error on input [rte.par]: xbmax =< xbmin'
         errcode = 5
         return
      end if

      if(ybmax .le. ybmin) then
         write(*,*) 'Error on input [rte.par]: ybmax =< ybmin'
         errcode = 6
         return
      end if

      IF(Domain .eq. 'VOL') THEN

         if(zbmax .le. zbmin) then
            write(*,*) 'Error on input [rte.par]: zbmax =< zbmin'
            errcode = 11
            return
         end if

         if(zbmax .gt. zsmax) then
            write(*,*) 'Error on input [rte.par]: zbmax > zsmax'
            errcode = 12
            return
         end if

         if(zbmin .lt. zsmin) then
            write(*,*) 'Error on input [rte.par]: zbmin < zsmin'
            errcode = 13
            return
         end if

      END IF

      if(lx8 .lt. 1) then
         write(*,*) 'Error on input [rte.par]: lx<1'
         errcode = 14
         return
      else if(lx8 .gt. i4_max) then
         write(*,*) 'Error on input [rte.par]: lx>i4_max=', i4_max
         errcode = 15
         return
      else
         lx  = lx8
         llx = log10(dfloat(lx)) + 1
      end if

      if(ly8 .lt. 1) then
         write(*,*) 'Error on input [rte.par]: ly<1'
         errcode = 16
         return
      else if(ly8 .gt. i4_max) then
         write(*,*) 'Error on input [rte.par]: ly>i4_max=', i4_max
         errcode = 17
         return
      else
         ly = ly8
         lly = log10(dfloat(ly)) + 1
      end if

      IF(Domain .eq. 'VOL') THEN

         if(lz8 .lt. 1) then
            write(*,*) 'Error on input [rte.par]: lz<1'
            errcode = 18
            return
         else if(lz8 .gt. i4_max) then
            write(*,*) 'Error on input [rte.par]: lz>i4_max=', i4_max
            errcode = 19
            return
         else
            lz = lz8
            llz = log10(dfloat(lz)) + 1
         end if

      END IF

c.......... (1.b.iv) Source ...................

      if(z0 .lt. zsmin) then
         write(*,*) 'Error on input [rte.par]: z0<zsmin'
         errcode = 24
         return
      else if(z0 .gt. zsmax) then
         write(*,*) 'Error on input [rte.par]: z0>zsmax'
         errcode = 25
         return
      end if

      s0 = sqrt(s0x*s0x + s0y*s0y + s0z*s0z)
      if(s0 .eq. 0.0) then
         write(*,*) 'Error on input [rte.par]: s0=0'
         errcode = 26
         return
      else
         s0x = s0x/s0
         s0y = s0y/s0
         s0z = s0z/s0
      end if

      if     (SourceType .eq. 'PNTC') then
         Collimated_Source = .true.
         Plane_Wave_Source = .false.
      else if(SourceType .eq. 'PNTI') then
         Collimated_Source = .false.
         Plane_Wave_Source = .false.
      else if(SourceType .eq. 'PLWC') then
         Collimated_Source = .true.
         Plane_Wave_Source = .true.
      else if(SourceType .eq. 'PLWI') then
         Collimated_Source = .false.
         Plane_Wave_Source = .true.
      else
         write(*,*) 'Error on input [rte.par]: incorrect SourceType: ', SourceType
	 errcode = 27
	 return
      end if

c.......... (1.b.v) Phase function ...................

      if     (PhaseFunction .eq. 'ISO') then
         if(g .ne. rzero) then
            write(*,*) '.....Warning on input [rte.par]:'
            write(*,*) '.....PhaseFunction=ISO but g=/=0'
            write(*,*) '.....I will set g=0 and continue'
            g = rzero
            errcode = -31
         end if
      else if(PhaseFunction .eq. 'HG') then
         if(g .eq. rzero) then
            write(*,*) '................Warning on input [rte.par]:'
            write(*,*) '...................PhaseFunction=HG but g=0'
            write(*,*) '..I will set PhaseFunction=ISO and continue'
            PhaseFunction = 'ISO'
            errcode = -32
         else if(abs(g) .lt. g_min) then
            write(*,*) '................Warning on input [rte.par]:'
            write(*,*) '............PhaseFunction=HG but 0<|g|<0.01'
            write(*,*) '..I will set PhaseFunction=HG0 and continue'
            PhaseFunction = 'HG0'
            errcode = -33
         end if
      else if(PhaseFunction .eq. 'HG0') then
         if(g .eq. rzero) then
            write(*,*) '................Warning on input [rte.par]:'
            write(*,*) '..................PhaseFunction=HG0 but g=0'
            write(*,*) '..I will set PhaseFunction=ISO and continue'
            PhaseFunction = 'ISO'
            errcode = -34
         else if(abs(g) .ge. g_min) then
            write(*,*) '................Warning on input [rte.par]:'
            write(*,*) '............PhaseFunction=HG0 but |g|>=0.01'
            write(*,*) '...I will set PhaseFunction=HG and continue'
            PhaseFunction = 'HG'
            errcode = -35
         end if
      else if(PhaseFunction .eq. 'EXP') then
         call make_beta(g, beta, info)
         if(info .ne. 0) then
            write(*,*) 'There were fatal errors computing beta(g); exiting. info=', info
            errcode = 38
            return
         else if(fun_exp(beta)-g .ne. rzero) then
            write(*,*) '.....Warning specific to PhaseFunction=EXP:'
            write(*,*) '..I was not able to compute beta precisely '
            write(*,*) '.........................fun_exp(beta)=/=g ' 
            write(*,*) 'g_actual, g_requested:', fun_exp(beta), g
            errcode = -36
         end if
         ebp = exp(beta)
         ebm = ru/ebp
         tshb = ebp - ebm
      else if(PhaseFunction .eq. 'RAY' ) then
         continue
	 write(*,*) 'PP RAY Not yet implemented...'
 	 pause 
      else if(PhaseFunction .eq. 'DEL' ) then
         continue
	 write(*,*) 'PP DEL Not yet implemented...'
 	 pause 
      else
         write(*,*) 'Error on input [rte.par]: incorrect PhaseFunction: ', PhaseFunction
	 errcode = 28
	 return
      end if

c.......... (1.b.vi) Computation mode ...................

      if((ncomp .lt. 0) .OR. (ncomp .gt. 5)) then
         write(*,*) 'Error on input [mc.par]: NCOMP outside of range'
         write(*,*) '        0 =< NCOMP =< 4                '
         errcode = 29
         return
      end if

      compute_u = .false.
      compute_J = .false.
      compute_I = .false.
      if((ncomp .eq. 1) .OR. (ncomp .eq. 2)) then
         compute_u = .true.
      end if
      if((ncomp .eq. 1) .OR. (ncomp .eq. 3)) then
         compute_J = .true.
      end if
      if(ncomp .eq. 4) then
         compute_I = .true.
      end if
      if(ncomp .eq. 5) then
         compute_u = .true.
         compute_J = .true.
         compute_I = .true.
      end if
      if(ncomp .eq. 0) then
         Compute_None = .true.
      end if

      IF(compute_I) THEN

         if(Nscth .lt. 3) then
            write(*,*) 'Error on input [rte.par]: Nscth<3'
     	    errcode = 36
	    return
         end if

         if(Nsphi .lt. 3) then
            write(*,*) 'Error on input [rte.par]: Nsphi<3'
     	    errcode = 37
	    return
         end if

         if(mod(Nscth,2) .eq. 0) then
            write(*,*) 'Warning [rte.par]: Nscth is even (it should probably be odd)'
     	    errcode = -39
         end if

         if(mod(Nsphi,2) .eq. 0) then
            write(*,*) 'Warning [rte.par]: Nsphi is even (it should probably be odd)'
     	    errcode = -40
         end if

      END IF


c.......... (1.b.vii) Units ...................

      if     (Units .eq. 'Ell_Star') then
         continue
      else if(Units .eq. 'Dog_Tail') then
         continue
      else
         write(*,*) 'Error on input [rte.par]: incorrect Units: ', Units
         errcode = 39
	 return
      end if
         
c.......... (1.b.viii) If Collimated_Source=F ...................

      IF(.not. Collimated_Source) THEN

         if(.not. Use_Symmetry) then

            write(*,*) 'Warning [rte.par]: Collimated_Source=F AND Use_Symmetry=F'
            write(*,*) '...... This does not make sense: the problem is symmetric'
            write(*,*) '...... I will set Use_Symmetry=T'
            Use_Symmetry = .true.
     	    errcode = -50

         end if

      END IF

c.......... (1.b.ix) If Use_Symmetry=T ...................

      IF(Use_Symmetry) THEN

         if(Collimated_Source) then

            if(abs(s0z-ru) .gt. small)  then
               write(*,*) 'Error on input [rte.par]: Use_Symmetry=Collimated_Source=T AND s0z=/=1'
               errcode = 45
  	       return
            else
               s0z = ru
               s0x = rzero
               s0y = rzero
            end if

         end if

      if(lrho8 .lt. 1) then
         write(*,*) 'Error on input [rte.par]: Use_Symmetry=T and lrho<1'
         errcode = 47
         return
      else if(lx8 .gt. i4_max) then
         write(*,*) 'Error on input [rte.par]: lrho>i4_max=', i4_max
         errcode = 48
         return
      else
         lrho  = lrho8
         llrho = log10(dfloat(lrho)) + 1
      end if

      END IF

c
c.......... (1.c) Reading MC parameters
c 

      open(unit=70,file='mc.par',status='old',err=2000)
         read(70,*,err=2001,end=2001) nmem_tot
         read(70,*,err=2002,end=2002) nw
         read(70,*,err=2003,end=2003) nr8
         read(70,*,err=2004,end=2004) nc8
         read(70,*,err=2005,end=2005) seed1, seed2, seed3, seed4
         read(70,*,err=2006,end=2006) seed5, seed6
      close(unit=70)

c
c......... (1.d) Checking MC parameters for sanity
c 

      if(nmem_tot .le. 0) then
         write(*,*) 'Error on input [mc.par]: nmem_tot<=0'
         errcode = 50
         return
      end if

      if(nw .le. 0) then
         write(*,*) 'Error on input [mc.par]: nw<=0'
         errcode = 51
         return
      end if

      if(nr8 .le. 2) then
         write(*,*) 'Error on input [mc.par]: nr<=2'
         errcode = 52
         return
      else if(nr8 .gt. i4_max) then
         write(*,*) 'Error on input [mc.par]: nr>i4_max=', i4_max
         errcode = 53
         return
      else if(nr8 .lt. 128) then
         write(*,*) 'Warning on input [mc.par]: nr<128'
         errcode = -1
      else
         nr = nr8
      end if

      if(nc8 .le. 0) then
         write(*,*) 'Error on input [mc.par]: nc<=0'
         errcode = 54
         return
      else if(nc8 .gt. i4_max) then
         write(*,*) 'Error on input [mc.par]: nc>i4_max=', i4_max
         errcode = 55
         return
      else
         nc = nc8
      end if

      if(seed1 .lt. 1e3) then
         write(*,*) 'Warning on input [mc.par]: seed1<1e4'
         errcode = -2
      end if
      if(seed2 .lt. 1e3) then
         write(*,*) 'Warning on input [mc.par]: seed2<1e4'
         errcode = -3
      end if
      if(seed3 .lt. 1e3) then
         write(*,*) 'Warning on input [mc.par]: seed3<1e4'
         errcode = -4
      end if
      if(seed4 .lt. 1e3) then
         write(*,*) 'Warning on input [mc.par]: seed4<1e4'
         errcode = -5
      end if

      IF(.not. Collimated_Source) THEN

      if(seed5 .lt. 1e3) then
         write(*,*) 'Warning on input [mc.par]: seed5<1e4'
         errcode = -6
      end if
      if(seed6 .lt. 1e3) then
         write(*,*) 'Warning on input [mc.par]: seed6<1e4'
         errcode = -7
      end if

      END IF
 

      if(iabs(seed1-seed2) .lt. 1e2) then
         write(*,*) 'Warning on input [mc.par]: |SEED1-SEED2|<100'
         errcode = -8
      end if
      if(iabs(seed1-seed3) .lt. 1e2) then
         write(*,*) 'Warning on input [mc.par]: |SEED1-SEED3|<100'
         errcode = -9
      end if
      if(iabs(seed1-seed4) .lt. 1e2) then
         write(*,*) 'Warning on input [mc.par]: |SEED1-SEED4|<100'
         errcode = -10
      end if
      if(iabs(seed2-seed3) .lt. 1e2) then
         write(*,*) 'Warning on input [mc.par]: |SEED2-SEED3|<100'
         errcode = -11
      end if
      if(iabs(seed2-seed4) .lt. 1e2) then
         write(*,*) 'Warning on input [mc.par]: |SEED2-SEED4|<100'
         errcode = -12
      end if
      if(iabs(seed3-seed4) .lt. 1e2) then
         write(*,*) 'Warning on input [mc.par]: |SEED3-SEED4|<100'
         errcode = -13
      end if

      if(.not. Collimated_Source) then

      if(iabs(seed1-seed5) .lt. 1e2) then
         write(*,*) 'Warning on input [mc.par]: |SEED1-SEED5|<100'
         errcode = -14
      end if
      if(iabs(seed1-seed6) .lt. 1e2) then
         write(*,*) 'Warning on input [mc.par]: |SEED1-SEED6|<100'
         errcode = -15
      end if
      if(iabs(seed2-seed5) .lt. 1e2) then
         write(*,*) 'Warning on input [mc.par]: |SEED2-SEED5|<100'
         errcode = -16
      end if
      if(iabs(seed2-seed6) .lt. 1e2) then
         write(*,*) 'Warning on input [mc.par]: |SEED2-SEED6|<100'
         errcode = -17
      end if
      if(iabs(seed3-seed5) .lt. 1e2) then
         write(*,*) 'Warning on input [mc.par]: |SEED3-SEED5|<100'
         errcode = -18
      end if
      if(iabs(seed3-seed6) .lt. 1e2) then
         write(*,*) 'Warning on input [mc.par]: |SEED3-SEED6|<100'
         errcode = -19
      end if
      if(iabs(seed4-seed5) .lt. 1e2) then
         write(*,*) 'Warning on input [mc.par]: |SEED4-SEED5|<100'
         errcode = -20
      end if
      if(iabs(seed4-seed6) .lt. 1e2) then
         write(*,*) 'Warning on input [mc.par]: |SEED4-SEED6|<100'
         errcode = -21
      end if
      if(iabs(seed5-seed6) .lt. 1e2) then
         write(*,*) 'Warning on input [mc.par]: |SEED5-SEED6|<100'
         errcode = -22
      end if

      end if

c
c---------- (2) Input-dependent parameters -----------------------------
c 

      mu_t     = mu_a + mu_s
      mu_star  = mu_a + (ru - g)*mu_s
      ell_star = ru/mu_star

      if     (Units .eq. 'Ell_Star') then
         scale = mu_t / (mu_a + (ru - g)*mu_s)
      else if(Units .eq. 'Dog_Tail') then
         scale = mu_t
         opt_depth = (zsmax - zsmin) / ell_star
      end if

      twog     = two*g
      gs       = g*g
      opgs     = ru + gs
      omgs     = ru - gs

      albedo   = dble(mu_a / mu_t)

      if(Use_Symmetry) then
         hrho = rhomax / float(lrho)
      else
         hx = (xbmax - xbmin) / float(lx)
         hy = (ybmax - ybmin) / float(ly)
      end if

      if(Domain .eq. 'VOL') then
         hz = (zbmax - zbmin) / float(lz)
      end if

      gmem_tot = float(nmem_tot)*nano

      sphi_min = rzero
      sphi_max = two
      if(Domain .eq. 'VOL') then
         scth_min = -ru
         scth_max =  ru
      else if(Domain .eq. 'SUR') then
         scth_min =  rzero
         scth_max =  ru
      end if 

      dscth    = (scth_max - scth_min)/float(Nscth)
      dsphi    = (sphi_max - sphi_min)/float(Nsphi-1)
      sphifrac = ru / dfloat(Nsphi-1)
      sphihrac = half*sphifrac

      if(Collimated_Source) then
         weight = ru
      else
         weight = pi4
      end if

c
c---------- (3) Populating the string header -----------------------------------
c 

      call make_string()

c
c-------- (4) VSL parameters --------------------------------------------------
c 

      brng1 = VSL_BRNG_MCG59
      meth1 = VSL_RNG_METHOD_EXPONENTIAL_ICDF
      vsl_errcode = vslnewstream(str1,brng1,seed1)
      call check_vsl_error(vsl_errcode)

      brng2 = VSL_BRNG_MCG31
      meth2 = VSL_RNG_METHOD_UNIFORM_STD
      vsl_errcode = vslnewstream(str2,brng2,seed2)
      call check_vsl_error(vsl_errcode)

      brng3 = VSL_BRNG_MT2203
      meth3 = VSL_RNG_METHOD_UNIFORM_STD
      vsl_errcode = vslnewstream(str3,brng3,seed3)
      call check_vsl_error(vsl_errcode)

      brng4 = VSL_BRNG_MCG59
      meth4 = VSL_RNG_METHOD_BERNOULLI_ICDF
      vsl_errcode = vslnewstream(str4,brng4,seed4)
      call check_vsl_error(vsl_errcode)

      if(.not. Collimated_Source) then

      brng5 = VSL_BRNG_MCG31
      meth5 = VSL_RNG_METHOD_UNIFORM_STD
      vsl_errcode = vslnewstream(str5,brng5,seed5)
      call check_vsl_error(vsl_errcode)

      brng6 = VSL_BRNG_MT2203
      meth6 = VSL_RNG_METHOD_UNIFORM_STD
      vsl_errcode = vslnewstream(str6,brng6,seed6)
      call check_vsl_error(vsl_errcode)

      end if

      RETURN

c
c-------- (5) Error code assignment ----------------------------------------------
c 

c-------- (5.a) Errors in rte.par ----------------------------------------------

 1000 continue
      write(*,*) 'Error openning file rte.par'
      errcode = 1000
      return

 1001 continue
      write(*,*) 'Error reading MU_A, MU_S [from rte.par]'
      errcode = 1001
      return

 1002 continue
      write(*,*) 'Error reading G [from rte.par]'
      errcode = 1002
      return

 1003 continue
      write(*,*) 'Error reading ZSMIN, ZSMAX [from rte.par]'
      errcode = 1003
      return

 1004 continue
      write(*,*) 'Error reading XBMIN, YBMIN, ZBMIN [from rte.par]'
      errcode = 1004
      return

 1005 continue
      write(*,*) 'Error reading XBMAX, YBMAX, ZBMAX [from rte.par]'
      errcode = 1005
      return

 1006 continue
      write(*,*) 'Error reading LX, LY, LZ [from mc.par]'
      errcode = 1006
      return

 1007 continue
      write(*,*) 'Error reading RHOMAX, LRHO [from rte.par]'
      errcode = 1007
      return

 1008 continue
      write(*,*) 'Error reading Z0 [from rte.par]'
      errcode = 1008
      return

 1009 continue
      write(*,*) 'Error reading S0X, S0Y, S0Z [from rte.par]'
      errcode = 1009
      return

 1010 continue
      write(*,*) 'Error reading NSPHI, NSPHI [from rte.par]'
      errcode = 1010
      return

 1011 continue
      write(*,*) 'Error reading NCOMP [from rte.par]'
      errcode = 1011
      return

 1012 continue
      write(*,*) 'Error reading SourceType [from rte.par]'
      errcode = 1012
      return

 1013 continue
      write(*,*) 'Error reading PhaseFunction [from rte.par]'
      errcode = 1013
      return

 1014 continue
      write(*,*) 'Error reading Domain [from rte.par]'
      errcode = 1014
      return

 1015 continue
      write(*,*) 'Error reading Units [from rte.par]'
      errcode = 1015
      return

 1016 continue
      write(*,*) 'Error reading Symmetry [from rte.par]'
      errcode = 1016
      return

c-------- (5.b) Errors in mc.par ----------------------------------------------

 2000 continue
      write(*,*) 'Error openning file mc.par'
      errcode = 2000
      return

 2001 continue
      write(*,*) 'Error reading NMEM_TOT [from mc.par]'
      errcode = 2001
      return

 2002 continue
      write(*,*) 'Error reading NW [from mc.par]'
      errcode = 2002
      return

 2003 continue
      write(*,*) 'Error reading NR [from mc.par]'
      errcode = 2003
      return

 2004 continue
      write(*,*) 'Error reading NC [from mc.par]'
      errcode = 2004
      return

 2005 continue
      write(*,*) 'Error reading SEED[1,2,3,4] [from mc.par]'
      write(*,*) 'Note: for Collimated_Sourcde=T, four seeds are requred'
      errcode = 2005
      return

 2006 continue
      write(*,*) 'Error reading SEED[1,2,3,4,5,6] [from mc.par]'
      write(*,*) 'Note: for Collimated_Source=F, six seeds are requred'
      errcode = 2006
      return

      end
